@@ -98,6 +98,13 @@ RsttoImageList *rstto_image_list_new ();
 gint     rstto_image_list_get_n_images (RsttoImageList *image_list);
 gboolean rstto_image_list_add_file (RsttoImageList *image_list, RsttoFile *file, GError **);
 
+gboolean
+rstto_image_list_set_directory (
+        RsttoImageList *image_list,
+        GFile *dir,
+        GError **);
+
+
 GCompareFunc rstto_image_list_get_compare_func (RsttoImageList *image_list);
 void         rstto_image_list_set_compare_func (RsttoImageList *image_list, GCompareFunc func);
 
@@ -107,22 +114,31 @@ rstto_image_list_set_sort_by_name (RsttoImageList *image_list);
 void
 rstto_image_list_set_sort_by_date (RsttoImageList *image_list);
 
-RsttoImageListIter *rstto_image_list_get_iter (RsttoImageList *image_list);
+RsttoImageListIter *
+rstto_image_list_get_iter (RsttoImageList *image_list);
 
 /** Iter functions */
-GType       rstto_image_list_iter_get_type ();
+GType
+rstto_image_list_iter_get_type ();
 
 RsttoFile *
 rstto_image_list_iter_get_file ( RsttoImageListIter *iter );
 
-void        rstto_image_list_iter_previous (RsttoImageListIter *iter);
-void        rstto_image_list_iter_next (RsttoImageListIter *iter);
+gboolean
+rstto_image_list_iter_previous (RsttoImageListIter *iter);
+gboolean
+rstto_image_list_iter_next (RsttoImageListIter *iter);
 
-gint        rstto_image_list_iter_get_position (RsttoImageListIter *iter);
-void        rstto_image_list_iter_set_position (RsttoImageListIter *iter, gint pos);
+gboolean
+rstto_image_list_iter_has_previous (RsttoImageListIter *iter);
+gboolean
+rstto_image_list_iter_has_next (RsttoImageListIter *iter);
 
+gint
+rstto_image_list_iter_get_position (RsttoImageListIter *iter);
 void
-rstto_image_list_remove_all (RsttoImageList *image_list);
+rstto_image_list_iter_set_position (RsttoImageListIter *iter, gint pos);
+
 void
 rstto_image_list_remove_file (RsttoImageList *image_list, RsttoFile *file);
 gboolean
@@ -130,6 +146,10 @@ rstto_image_list_iter_find_file (RsttoImageListIter *iter, RsttoFile *file);
 
 RsttoImageListIter *rstto_image_list_iter_clone (RsttoImageListIter *iter);
 
+gboolean
+rstto_image_list_iter_get_sticky (
+        RsttoImageListIter *iter);
+
 G_END_DECLS
 
 #endif /* __RISTRETTO_IMAGE_LIST_H__ */
