@@ -188,6 +188,9 @@ static void
 cb_rstto_image_loader_closed (GdkPixbufLoader *loader, RsttoImageViewerTransaction *transaction);
 static gboolean
 cb_rstto_image_viewer_update_pixbuf (RsttoImageViewer *viewer);
+static void
+cb_rstto_image_viewer_dnd (GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data,
+                           guint info, guint time_, RsttoImageViewer *viewer);
 
 static gboolean
 rstto_scroll_event (
@@ -307,6 +310,11 @@ rstto_image_viewer_init ( GObject *object )
             "notify::revert-zoom-direction",
             G_CALLBACK (cb_rstto_zoom_direction_changed),
             viewer);
+    g_signal_connect (
+            G_OBJECT(viewer),
+            "drag-data-received",
+            G_CALLBACK (cb_rstto_image_viewer_dnd),
+            viewer);
 
     gtk_widget_set_events (GTK_WIDGET(viewer),
                            GDK_BUTTON_PRESS_MASK |
@@ -315,10 +323,11 @@ rstto_image_viewer_init ( GObject *object )
                            GDK_ENTER_NOTIFY_MASK |
                            GDK_POINTER_MOTION_MASK);
 
-    /*
-    gtk_drag_dest_set(GTK_WIDGET(viewer), 0, drop_targets, G_N_ELEMENTS(drop_targets),
+
+    gtk_drag_dest_set(GTK_WIDGET(viewer), GTK_DEST_DEFAULT_ALL, NULL, 0,
                       GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE | GDK_ACTION_PRIVATE);
-    */
+    gtk_drag_dest_add_uri_targets (GTK_WIDGET(viewer));
+
 }
 
 /**
@@ -363,6 +372,29 @@ rstto_image_viewer_class_init(RsttoImageViewerClass *viewer_class)
                                 G_TYPE_NONE, 2,
                                 GTK_TYPE_ADJUSTMENT,
                                 GTK_TYPE_ADJUSTMENT);
+    g_signal_new (
+            "size-ready",
+            G_TYPE_FROM_CLASS (object_class),
+            G_SIGNAL_RUN_FIRST,
+            0,
+            NULL, NULL,
+            g_cclosure_marshal_VOID__VOID,
+            G_TYPE_NONE, 0);
+    g_signal_new ("scale-changed",
+            G_TYPE_FROM_CLASS (object_class),
+            G_SIGNAL_RUN_FIRST,
+            0,
+            NULL, NULL,
+            g_cclosure_marshal_VOID__VOID,
+            G_TYPE_NONE, 0);
+    g_signal_new ("files-dnd",
+            G_TYPE_FROM_CLASS (object_class),
+            G_SIGNAL_RUN_FIRST,
+            0,
+            NULL, NULL,
+            g_cclosure_marshal_VOID__POINTER,
+            G_TYPE_NONE, 1,
+            G_TYPE_POINTER);
 }
 
 /**
@@ -529,15 +561,25 @@ rstto_image_viewer_destroy(GtkObject *object)
 {
     RsttoImageViewer *viewer = RSTTO_IMAGE_VIEWER(object);
 
-    if (viewer->priv->settings)
+    if (viewer->priv)
     {
-        g_object_unref (viewer->priv->settings);
-        viewer->priv->settings = NULL;
-    }
-    if (viewer->priv->bg_icon)
-    {
-        g_object_unref (viewer->priv->bg_icon);
-        viewer->priv->bg_icon = NULL;
+        if (viewer->priv->settings)
+        {
+            g_object_unref (viewer->priv->settings);
+            viewer->priv->settings = NULL;
+        }
+        if (viewer->priv->bg_icon)
+        {
+            g_object_unref (viewer->priv->bg_icon);
+            viewer->priv->bg_icon = NULL;
+        }
+        if (viewer->priv->pixbuf)
+        {
+            g_object_unref (viewer->priv->pixbuf);
+            viewer->priv->pixbuf = NULL;
+        }
+        g_free (viewer->priv);
+        viewer->priv = NULL;
     }
 }
 
@@ -724,6 +766,7 @@ set_scale (
 
     viewer->priv->auto_scale = auto_scale;
     viewer->priv->scale = scale;
+    g_signal_emit_by_name(viewer, "scale-changed");
 }
  
 static void
@@ -798,12 +841,15 @@ paint_background_icon (
 
     /* Draw the pixbuf on the cairo-context */
     /****************************************/
-    gdk_cairo_set_source_pixbuf (
-            ctx,
-            viewer->priv->bg_icon,
-            0.0,
-            0.0);
-    cairo_paint_with_alpha (ctx, 0.1);
+    if(viewer->priv->bg_icon != NULL)
+    {
+        gdk_cairo_set_source_pixbuf (
+                ctx,
+                viewer->priv->bg_icon,
+                0.0,
+                0.0);
+        cairo_paint_with_alpha (ctx, 0.1);
+    }
 }
 
 static void
@@ -920,10 +966,10 @@ paint_image (
             case RSTTO_IMAGE_ORIENT_270:
                 viewer->priv->rendering.x_offset = ((gdouble)widget->allocation.width - (
                             (gdouble)viewer->priv->image_height * 
-                                viewer->priv->scale) ) / 2.0+0.5;
+                                viewer->priv->scale) ) / 2.0;
                 viewer->priv->rendering.y_offset = ((gdouble)widget->allocation.height - (
                             (gdouble)viewer->priv->image_width * 
-                                viewer->priv->scale) ) / 2.0+0.5;
+                                viewer->priv->scale) ) / 2.0;
                 viewer->priv->rendering.width = 
                         (gdouble)viewer->priv->image_height * viewer->priv->scale;
                 viewer->priv->rendering.height = 
@@ -934,10 +980,10 @@ paint_image (
             default:
                 viewer->priv->rendering.x_offset = ((gdouble)widget->allocation.width - (
                             (gdouble)viewer->priv->image_width * 
-                                viewer->priv->scale) ) / 2.0+0.5;
+                                viewer->priv->scale) ) / 2.0;
                 viewer->priv->rendering.y_offset = ((gdouble)widget->allocation.height - (
                             (gdouble)viewer->priv->image_height * 
-                                viewer->priv->scale) ) / 2.0+0.5;
+                                viewer->priv->scale) ) / 2.0;
                 viewer->priv->rendering.width = 
                         (gdouble)viewer->priv->image_width * viewer->priv->scale;
                 viewer->priv->rendering.height = 
@@ -946,13 +992,13 @@ paint_image (
 
         }
 
-        if (viewer->priv->rendering.x_offset < 0.5)
+        if (viewer->priv->rendering.x_offset < 0)
         {
-            viewer->priv->rendering.x_offset = 0.5;
+            viewer->priv->rendering.x_offset = 0;
         }
-        if (viewer->priv->rendering.y_offset < 0.5)
+        if (viewer->priv->rendering.y_offset < 0)
         {
-            viewer->priv->rendering.y_offset = 0.5;
+            viewer->priv->rendering.y_offset = 0;
         }
 
         cairo_save (ctx);
@@ -1165,6 +1211,21 @@ paint_selection_box (
         box_height = (y_offset + image_height) - box_y - 1;
     }
 
+    /* Make sure the box dimensions are not negative,
+     * This results in rendering-artifacts.
+     */
+    if (box_width < 0.0)
+    {
+        /* Return, do not draw the box.  */
+        return;
+    }
+    /* Same as above, for the vertical dimensions this time */
+    if (box_height < 0.0)
+    {
+        /* Return, do not draw the box.  */
+        return;
+    }
+
     cairo_rectangle (
         ctx,
         box_x+0.5, box_y+0.5,
@@ -1546,6 +1607,25 @@ rstto_image_viewer_get_orientation (RsttoImageViewer *viewer)
     return viewer->priv->orientation;
 }
 
+gint
+rstto_image_viewer_get_width (RsttoImageViewer *viewer)
+{
+    if (viewer)
+    {
+        return viewer->priv->image_width;
+    }
+    return 0;
+}
+gint
+rstto_image_viewer_get_height (RsttoImageViewer *viewer)
+{
+    if (viewer)
+    {
+        return viewer->priv->image_height;
+    }
+    return 0;
+}
+
 void
 rstto_image_viewer_set_menu (
     RsttoImageViewer *viewer,
@@ -1780,6 +1860,7 @@ cb_rstto_image_loader_closed (GdkPixbufLoader *loader, RsttoImageViewerTransacti
                 FALSE);
     }
 
+    g_signal_emit_by_name(transaction->viewer, "size-ready");
     rstto_image_viewer_transaction_free (transaction);
 }
 
@@ -2013,6 +2094,8 @@ rstto_scroll_event (
                     widget->window,
                     NULL,
                     FALSE); 
+
+            g_signal_emit_by_name(viewer, "scale-changed");
         }
         return TRUE;
     }
@@ -2076,6 +2159,25 @@ rstto_motion_notify_event (
                         widget->window,
                         NULL,
                         FALSE); 
+
+                /* Only change the cursor when hovering over the image
+                 */
+                if ( (event->x > viewer->priv->rendering.x_offset) &&
+                     (event->y > viewer->priv->rendering.y_offset) &&
+                     (event->y < (viewer->priv->rendering.y_offset + viewer->priv->rendering.height)) &&
+                     (event->x < (viewer->priv->rendering.x_offset + viewer->priv->rendering.width)))
+                {
+                    GdkCursor *cursor = gdk_cursor_new(GDK_UL_ANGLE);
+                    gdk_window_set_cursor(widget->window, cursor);
+                    gdk_cursor_unref(cursor);
+                }
+                else
+                {
+                    /* Set back to default when moving over the
+                     * background.
+                     */
+                    gdk_window_set_cursor(widget->window, NULL);
+                }
                 break;
             default:
                 break;
@@ -2104,18 +2206,37 @@ rstto_button_press_event (
         {
             if (!(event->state & (GDK_CONTROL_MASK)))
             {
-                GdkCursor *cursor = gdk_cursor_new(GDK_FLEUR);
-                gdk_window_set_cursor(widget->window, cursor);
-                gdk_cursor_unref(cursor);
-                rstto_image_viewer_set_motion_state (viewer, RSTTO_IMAGE_VIEWER_MOTION_STATE_MOVE);
+                if ( (event->x > viewer->priv->rendering.x_offset) &&
+                     (event->y > viewer->priv->rendering.y_offset) &&
+                     (event->y < (viewer->priv->rendering.y_offset + viewer->priv->rendering.height)) &&
+                     (event->x < (viewer->priv->rendering.x_offset + viewer->priv->rendering.width)))
+                {
+                    GdkCursor *cursor = gdk_cursor_new(GDK_FLEUR);
+                    gdk_window_set_cursor(widget->window, cursor);
+                    gdk_cursor_unref(cursor);
+                    rstto_image_viewer_set_motion_state (viewer, RSTTO_IMAGE_VIEWER_MOTION_STATE_MOVE);
+                }
             }
 
             if (event->state & GDK_CONTROL_MASK)
             {
-                GdkCursor *cursor = gdk_cursor_new(GDK_UL_ANGLE);
-                gdk_window_set_cursor(widget->window, cursor);
-                gdk_cursor_unref(cursor);
+                /* Only change the cursor when hovering over the image
+                 */
+                if ( (event->x > viewer->priv->rendering.x_offset) &&
+                     (event->y > viewer->priv->rendering.y_offset) &&
+                     (event->y < (viewer->priv->rendering.y_offset + viewer->priv->rendering.height)) &&
+                     (event->x < (viewer->priv->rendering.x_offset + viewer->priv->rendering.width)))
+                {
+                    GdkCursor *cursor = gdk_cursor_new(GDK_UL_ANGLE);
+                    gdk_window_set_cursor(widget->window, cursor);
+                    gdk_cursor_unref(cursor);
+                }
 
+                /* Set the zoom-state even if not hovering over the
+                 * image, this allows for easier selection.
+                 * Dragging from / to somewhere outside the image to
+                 * make sure the border is selected too.
+                 */
                 rstto_image_viewer_set_motion_state (viewer, RSTTO_IMAGE_VIEWER_MOTION_STATE_BOX_ZOOM);
             }
         }
@@ -2384,3 +2505,26 @@ rstto_popup_menu (
     }
     return FALSE;
 }
+
+static void
+cb_rstto_image_viewer_dnd (GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data,
+              guint info, guint time_, RsttoImageViewer *viewer)
+{
+    g_return_if_fail ( RSTTO_IS_IMAGE_VIEWER(viewer) );
+
+    if ((data->length >= 0) && (data->format == 8))
+    {
+        gchar **uris;
+
+        uris = g_uri_list_extract_uris ((const gchar*)data->data);
+
+        g_signal_emit_by_name (viewer, "files-dnd", uris);
+
+        gtk_drag_finish (context, TRUE, FALSE, time_);
+        g_strfreev (uris);
+    }
+    else
+    {
+        gtk_drag_finish (context, FALSE, FALSE, time_);
+    }
+}
