@@ -24,17 +24,29 @@
 
 #include "monitor_chooser.h"
 
+#define RSTTO_MAX_MONITORS 9
+
 typedef struct {
     gint width;
     gint height;
-    GdkPixbuf *pixbuf;
-    RsttoMonitorStyle style;
+
+    cairo_surface_t *image_surface;
 } Monitor;
 
+typedef struct {
+    guint x;
+    guint y;
+    guint width;
+    guint height;
+} MonitorPosition;
+
 struct _RsttoMonitorChooserPriv
 {
-    GSList *monitors;
+    Monitor **monitors;
+    gint n_monitors;
     gint selected;
+
+    MonitorPosition monitor_positions[RSTTO_MAX_MONITORS];
 };
 
 static GtkWidgetClass *parent_class = NULL;
@@ -63,7 +75,8 @@ static void
 cb_rstto_button_press_event (GtkWidget *, GdkEventButton *event);
 
 static void
-paint_monitor ( cairo_t *cr,
+paint_monitor ( GtkWidget *widget,
+                cairo_t *cr,
                 gdouble x,
                 gdouble y,
                 gdouble width,
@@ -112,6 +125,7 @@ rstto_monitor_chooser_init(RsttoMonitorChooser *chooser)
 {
     chooser->priv = g_new0(RsttoMonitorChooserPriv, 1);
     chooser->priv->selected = -1;
+    chooser->priv->monitors = g_new0 (Monitor *, 1);
 
     gtk_widget_set_double_buffered (GTK_WIDGET(chooser), TRUE);
 
@@ -226,9 +240,9 @@ static gboolean
 rstto_monitor_chooser_paint(GtkWidget *widget)
 {
     RsttoMonitorChooser *chooser = RSTTO_MONITOR_CHOOSER (widget);
-    GSList *iter = chooser->priv->monitors;
     cairo_t *ctx = gdk_cairo_create (widget->window);
     Monitor *monitor;
+
     gdouble width, height;
     gchar *label = NULL;
     gint row_width = 0;
@@ -245,26 +259,30 @@ rstto_monitor_chooser_paint(GtkWidget *widget)
             (gdouble)widget->allocation.height);
     cairo_fill (ctx);
 
-    if (g_slist_length (chooser->priv->monitors) > 1)
+    if (chooser->priv->n_monitors > 1)
     {
-        while (NULL != iter)
+        for (; chooser->priv->monitors[id]; ++id)
         {
-            monitor = iter->data;
+            monitor = chooser->priv->monitors[id];
+
             /* Render the selected monitor a little bigger */
-            if (chooser->priv->selected == g_slist_index (chooser->priv->monitors, monitor))
+            if (id == chooser->priv->selected)
             {
                 if (monitor->width > monitor->height)
                 {
                     width = widget->allocation.width*0.4;
-                    height = (gdouble)width / (gdouble)monitor->width * (gdouble)monitor->height;
+                    height = width;
                 } 
                 else
                 {
                     height = widget->allocation.width*0.4;
-                    width = (gdouble)height / (gdouble)monitor->height * (gdouble)monitor->width;
+                    width = height;
                 }
-                label = g_strdup_printf("%d", g_slist_index (chooser->priv->monitors, monitor)+1);
-                paint_monitor (ctx,
+                label = g_strdup_printf("%d", id+1);
+                cairo_save (ctx);
+                paint_monitor (
+                        widget,
+                        ctx,
                         ((gdouble)widget->allocation.width/4) - (width/2.0),
                         ((gdouble)widget->allocation.height - height)/2.0,
                         width,
@@ -272,61 +290,87 @@ rstto_monitor_chooser_paint(GtkWidget *widget)
                         label,
                         monitor,
                         TRUE);
+                cairo_restore (ctx);
                 g_free (label);
             }
             else
             {
-                row_width = sqrt (g_slist_length(chooser->priv->monitors));
+                row_width = sqrt (chooser->priv->n_monitors);
 
                 if (monitor->width > monitor->height)
                 {
                     width = widget->allocation.width*(0.4/((gdouble)row_width+1));
-                    height = (gdouble)width / (gdouble)monitor->width * (gdouble)monitor->height;
+                    height = width;
                 } 
                 else
                 {
-                    height = widget->allocation.width*(0.4/g_slist_length(chooser->priv->monitors));
-                    width = (gdouble)height / (gdouble)monitor->height * (gdouble)monitor->width;
+                    height = widget->allocation.width*(0.4/chooser->priv->n_monitors);
+                    width = height;
                 }
             
 
-                label = g_strdup_printf("%d", g_slist_index (chooser->priv->monitors, monitor)+1);
-                paint_monitor (ctx,
-                        ((gdouble)widget->allocation.width/2)+
-                            (((gdouble)widget->allocation.width/2)/
-                            (row_width+1))*(id%(row_width)+1)-
-                            (width/2.0),
-                        ((gdouble)widget->allocation.height/
-                            (row_width+2)*(id/row_width+1))-
-                            (height/2.0),
-                        width,
-                        height,
-                        label,
-                        monitor,
-                        FALSE);
+                label = g_strdup_printf("%d", id+1);
+                cairo_save (ctx);
+                if (id < chooser->priv->selected)
+                {
+                    paint_monitor (
+                            widget,
+                            ctx,
+                            ((gdouble)widget->allocation.width/2)+
+                                (((gdouble)widget->allocation.width/2)/
+                                (row_width+1))*(id%(row_width)+1)-
+                                (width/2.0),
+                            ((gdouble)widget->allocation.height/
+                                (row_width+2)*(id/row_width+1))-
+                                (height/2.0),
+                            width,
+                            height,
+                            label,
+                            monitor,
+                            FALSE);
+                }
+                else
+                {
+                    paint_monitor (
+                            widget,
+                            ctx,
+                            ((gdouble)widget->allocation.width/2)+
+                                (((gdouble)widget->allocation.width/2)/
+                                (row_width+1))*((id-1)%(row_width)+1)-
+                                (width/2.0),
+                            ((gdouble)widget->allocation.height/
+                                (row_width+2)*((id-1)/row_width+1))-
+                                (height/2.0),
+                            width,
+                            height,
+                            label,
+                            monitor,
+                            FALSE);
+                }
+                cairo_restore (ctx);
                 g_free (label);
-
-                id++;
             }
-            iter = g_slist_next(iter);
         }
     }
     else
     {
-        if (iter)
+        if (chooser->priv->monitors[0])
         {
-            monitor = iter->data;
+            monitor = chooser->priv->monitors[0];
             if (monitor->width > monitor->height)
             {
                 width = 200;
-                height = (gdouble)width / (gdouble)monitor->width * (gdouble)monitor->height;
+                height = 200;
             } 
             else
             {
                 height = 200;
-                width = (gdouble)height / (gdouble)monitor->height * (gdouble)monitor->width;
+                width = 200;
             }
-            paint_monitor (ctx,
+            cairo_save (ctx);
+            paint_monitor (
+                    widget,
+                    ctx,
                     ((gdouble)widget->allocation.width - width)/2.0,
                     ((gdouble)widget->allocation.height - height)/2.0,
                     width,
@@ -334,13 +378,18 @@ rstto_monitor_chooser_paint(GtkWidget *widget)
                     "1",
                     monitor,
                     TRUE);
+            cairo_restore (ctx);
         }
     }
+
+    cairo_destroy (ctx);
+
     return FALSE;
 }
 
 static void
-paint_monitor ( cairo_t *cr,
+paint_monitor ( GtkWidget *widget,
+                cairo_t *cr,
                 gdouble x,
                 gdouble y,
                 gdouble width,
@@ -349,17 +398,39 @@ paint_monitor ( cairo_t *cr,
                 Monitor *monitor,
                 gboolean active)
 {
-    gdouble line_width = 3.0;
-    double corner_radius = height / 10;
-    double radius = corner_radius / 1.0;
+    /* Do we want the border_padding to be a percentage of the width
+     * parmeter?
+     */
+    gdouble border_padding = 10.0;
+    gdouble foot_height = height / 10.0;
+    gdouble monitor_border_width = height * 0.04;
+
+    /* Assumption: monitor-width is always larger then monitor-height */
+    /******************************************************************/
+    gdouble monitor_height = 
+            ( width - ( 2 * ( border_padding + monitor_border_width ) ) ) /
+            monitor->width * monitor->height;
+    gdouble monitor_width = 
+            width - ( 2 * ( border_padding + monitor_border_width ) );
+
+    gdouble monitor_x = x + border_padding + monitor_border_width;
+
+    gdouble monitor_y = y +
+            ( height -
+                    ( ( border_padding + monitor_border_width ) * 2 ) -
+                    foot_height - monitor_height);
+
+    gdouble line_width = 2.0;
+    double radius = monitor_border_width * 0.5;
     double degrees = M_PI / 180.0;
     gint text_width = 0.0;
     gint text_height = 0.0;
     gdouble hscale = 1.0;
     gdouble vscale = 1.0;
+
+    /*******************************************/
     PangoLayout *layout;
     PangoFontDescription *font_description;
-    GdkPixbuf *dst_pixbuf = NULL;
 
     g_return_if_fail (NULL != monitor);
     
@@ -367,118 +438,137 @@ paint_monitor ( cairo_t *cr,
      * Set path for monitor outline and background-color.
      */
     cairo_new_sub_path (cr);
-    cairo_arc (cr, x + width - radius, y + radius, radius, -90 * degrees, 0 * degrees);
-    cairo_arc (cr, x + width - radius, y + height - radius, radius, 0 * degrees, 90 * degrees);
-    cairo_arc (cr, x + radius, y + height - radius, radius, 90 * degrees, 180 * degrees);
-    cairo_arc (cr, x + radius, y + radius, radius, 180 * degrees, 270 * degrees);
+    cairo_arc (
+            cr,
+            monitor_x + monitor_width + monitor_border_width - radius,
+            monitor_y - monitor_border_width + radius,
+            radius,
+            -90 * degrees,
+            0);
+    cairo_arc (
+            cr,
+            monitor_x + monitor_width + monitor_border_width - radius,
+            monitor_y + monitor_height + monitor_border_width - radius,
+            radius,
+            0,
+            90 * degrees);
+    cairo_arc (
+            cr,
+            monitor_x - monitor_border_width + radius,
+            monitor_y + monitor_height + monitor_border_width - radius,
+            radius,
+            90 * degrees,
+            180 * degrees);
+    cairo_arc (
+            cr,
+            monitor_x - monitor_border_width + radius,
+            monitor_y - monitor_border_width + radius,
+            radius,
+            180 * degrees,
+            270 * degrees);
     cairo_close_path (cr);
 
     /* Fill the background-color */
-    cairo_set_source_rgb (cr, 0.9, 0.9, 0.9);
+    gdk_cairo_set_source_color (
+            cr,
+            &(widget->style->base[GTK_STATE_NORMAL]));
     cairo_fill_preserve (cr);
 
     /* Paint the outside border */
-    cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 1.0);
+    gdk_cairo_set_source_color (
+            cr,
+            &(widget->style->fg[GTK_STATE_NORMAL]));
     cairo_set_line_width (cr, line_width);
     cairo_stroke (cr);
 
+    /* Draw a monitor foot */ 
     cairo_new_sub_path (cr);
-    cairo_arc (cr, x + width - radius-line_width, y + radius+line_width, radius, -90 * degrees, 0 * degrees);
-    cairo_arc (cr, x + width - radius-line_width, y + height - radius-line_width, radius, 0 * degrees, 90 * degrees);
-    cairo_arc (cr, x + radius+line_width, y + height - radius-line_width, radius, 90 * degrees, 180 * degrees);
-    cairo_arc (cr, x + radius+line_width, y + radius+line_width, radius, 180 * degrees, 270 * degrees);
+    cairo_move_to (
+            cr,
+            monitor_x+(monitor_width-foot_height)/2.0,
+            monitor_y+(monitor_height+monitor_border_width));
+    cairo_line_to (
+            cr,
+            monitor_x+(monitor_width-foot_height)/2.0+foot_height,
+            monitor_y+(monitor_height+monitor_border_width));
+    cairo_line_to (
+            cr,
+            monitor_x+(monitor_width-foot_height)/2.0+foot_height,
+            monitor_y+(monitor_height+monitor_border_width+foot_height*0.5));
+    cairo_line_to (
+            cr,
+            monitor_x+(monitor_width-foot_height)/2.0+foot_height*2,
+            monitor_y+(monitor_height+monitor_border_width+foot_height*0.5));
+    cairo_line_to (
+            cr,
+            monitor_x+(monitor_width-foot_height)/2.0+foot_height*2,
+            monitor_y+(monitor_height+monitor_border_width+foot_height));
+    cairo_line_to (
+            cr,
+            monitor_x+(monitor_width-foot_height)/2.0-foot_height,
+            monitor_y+(monitor_height+monitor_border_width+foot_height));
+    cairo_line_to (
+            cr,
+            monitor_x+(monitor_width-foot_height)/2.0-foot_height,
+            monitor_y+(monitor_height+monitor_border_width+foot_height*0.5));
+    cairo_line_to (
+            cr,
+            monitor_x+(monitor_width-foot_height)/2.0,
+            monitor_y+(monitor_height+monitor_border_width+foot_height*0.5));
     cairo_close_path (cr);
+    gdk_cairo_set_source_color (
+            cr,
+            &(widget->style->base[GTK_STATE_NORMAL]));
+    cairo_fill_preserve (cr);
+    gdk_cairo_set_source_color (
+            cr,
+            &(widget->style->fg[GTK_STATE_NORMAL]));
+    cairo_set_line_width (cr, line_width);
+    cairo_stroke (cr);
 
-    /* Paint the inside border */
-    cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 1.0);
+    /* Draw a line around the image */
+    cairo_new_sub_path (cr);
+    cairo_move_to (cr, monitor_x, monitor_y);
+    cairo_line_to (cr, monitor_x+monitor_width, monitor_y);
+    cairo_line_to (cr, monitor_x+monitor_width, monitor_y + monitor_height);
+    cairo_line_to (cr, monitor_x, monitor_y + monitor_height);
+    cairo_close_path (cr);
+    gdk_cairo_set_source_color (
+            cr,
+            &(widget->style->fg[GTK_STATE_NORMAL]));
     cairo_set_line_width (cr, line_width);
     cairo_stroke (cr);
 
     /* Set the path that limits the image-size */
     cairo_new_sub_path (cr);
-    cairo_move_to (cr, x + height / 10.0, y + height / 10.0);
-    cairo_line_to (cr, x + width - height / 10, y + height / 10.0);
-    cairo_line_to (cr, x + width - height / 10, y + height - height / 10.0);
-    cairo_line_to (cr, x + height / 10.0, y + height - height / 10.0);
+    cairo_move_to (cr, monitor_x, monitor_y);
+    cairo_line_to (cr, monitor_x+monitor_width, monitor_y);
+    cairo_line_to (cr, monitor_x+monitor_width, monitor_y + monitor_height);
+    cairo_line_to (cr, monitor_x, monitor_y + monitor_height);
     cairo_close_path (cr);
 
     /* Color the background black */
     cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 1.0);
     cairo_fill_preserve (cr);
 
-    cairo_clip_preserve (cr);
-    if (monitor->pixbuf)
+    if (monitor->image_surface)
     {
-        switch (monitor->style)
-        {
-            case MONITOR_STYLE_STRETCHED:
-                hscale = (width) / (gdk_pixbuf_get_width(monitor->pixbuf));
-                vscale = (height) / (gdk_pixbuf_get_height(monitor->pixbuf));
-                cairo_scale (cr, hscale, vscale);
+        cairo_clip_preserve (cr);
 
-                gdk_cairo_set_source_pixbuf (cr,
-                                             monitor->pixbuf,
-                                             x/hscale+((width/hscale)-gdk_pixbuf_get_width(monitor->pixbuf))/2,
-                                             y/vscale+((height/vscale)-gdk_pixbuf_get_height(monitor->pixbuf))/2);
-                break;
-            case MONITOR_STYLE_SCALED:
-                hscale = (width) / (gdk_pixbuf_get_width(monitor->pixbuf));
-                vscale = (height) / (gdk_pixbuf_get_height(monitor->pixbuf));
-                if (hscale < vscale)
-                {
-                    vscale = hscale;
-                }
-                else
-                {
-                    hscale = vscale;
-                }
-                cairo_scale (cr, hscale, vscale);
+        hscale = monitor_width / (cairo_image_surface_get_width(monitor->image_surface));
+        vscale = monitor_height / (cairo_image_surface_get_height(monitor->image_surface));
 
-                gdk_cairo_set_source_pixbuf (cr,
-                                             monitor->pixbuf,
-                                             x/hscale+((width/hscale)-gdk_pixbuf_get_width(monitor->pixbuf))/2,
-                                             y/vscale+((height/vscale)-gdk_pixbuf_get_height(monitor->pixbuf))/2);
+        cairo_scale (cr, hscale, vscale);
 
-                break;
-            case MONITOR_STYLE_ZOOMED:
-                hscale = (width) / (gdk_pixbuf_get_width(monitor->pixbuf));
-                vscale = (height) / (gdk_pixbuf_get_height(monitor->pixbuf));
-                if (hscale > vscale)
-                {
-                    vscale = hscale;
-                }
-                else
-                {
-                    hscale = vscale;
-                }
-                cairo_scale (cr, hscale, vscale);
+        cairo_set_source_surface (
+                cr,
+                monitor->image_surface,
+                monitor_x/hscale,
+                monitor_y/vscale);
+        cairo_paint(cr);
 
-                gdk_cairo_set_source_pixbuf (cr,
-                                             monitor->pixbuf,
-                                             x/hscale+((width/hscale)-gdk_pixbuf_get_width(monitor->pixbuf))/2,
-                                             y/vscale+((height/vscale)-gdk_pixbuf_get_height(monitor->pixbuf))/2);
-                break;
-            default:
-                dst_pixbuf = gdk_pixbuf_copy (monitor->pixbuf);
-                gdk_pixbuf_saturate_and_pixelate (monitor->pixbuf, dst_pixbuf, 0.0, TRUE);
-                hscale = width / (gdk_pixbuf_get_width(monitor->pixbuf));
-                vscale = hscale;
-                cairo_scale (cr, hscale, vscale);
-
-                gdk_cairo_set_source_pixbuf (cr,
-                                             dst_pixbuf,
-                                             x/hscale+((width/hscale)-gdk_pixbuf_get_width(monitor->pixbuf))/2,
-                                             y/vscale+((height/vscale)-gdk_pixbuf_get_height(monitor->pixbuf))/2);
-                break;
-        }
-    }
-    cairo_paint(cr);
-    cairo_reset_clip(cr);
-    cairo_scale (cr, 1/hscale, 1/vscale);
-    if (NULL != dst_pixbuf)
-    {
-        g_object_unref (dst_pixbuf);
-        dst_pixbuf = NULL;
+        cairo_reset_clip(cr);
+        cairo_scale (cr, 1/hscale, 1/vscale);
     }
 
     if (FALSE == active)
@@ -488,14 +578,14 @@ paint_monitor ( cairo_t *cr,
     }
 
     cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 1.0);
-    cairo_set_line_width (cr, 2.0);
+    cairo_set_line_width (cr, line_width);
     cairo_stroke (cr);
 
 
     font_description = pango_font_description_new ();
     pango_font_description_set_family (font_description, "sans");
     pango_font_description_set_weight (font_description, PANGO_WEIGHT_BOLD);
-    pango_font_description_set_absolute_size (font_description, height*0.4 * PANGO_SCALE);
+    pango_font_description_set_absolute_size (font_description, height*0.3 * PANGO_SCALE);
 
 
     layout = pango_cairo_create_layout (cr);
@@ -505,9 +595,16 @@ paint_monitor ( cairo_t *cr,
     pango_cairo_update_layout (cr, layout);
     pango_layout_get_pixel_size (layout, &text_width, &text_height); 
 
-    cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.9);
-    cairo_move_to (cr, x+(width-(gdouble)text_width) / 2, y+(height/2 + ((gdouble)text_height / 3)));
-    pango_cairo_show_layout_line (cr, pango_layout_get_line (layout, 0));
+    cairo_move_to (
+            cr,
+            monitor_x+(monitor_width-(gdouble)text_width) / 2,
+            monitor_y+(monitor_height- (gdouble)text_height) / 2);
+    pango_cairo_layout_path (cr, layout);
+    cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.8);
+    cairo_fill_preserve (cr);
+    cairo_set_line_width (cr, line_width);
+    cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.9);
+    cairo_stroke (cr);
 
     g_object_unref (layout);
     pango_font_description_free (font_description);
@@ -529,37 +626,59 @@ rstto_monitor_chooser_add (
         gint width,
         gint height)
 {
+    Monitor **monitors = g_new0 (Monitor *, chooser->priv->n_monitors+2);
+    gint id = 0;
+
     Monitor *monitor = g_new0 (Monitor, 1);
     monitor->width = width;
     monitor->height = height;
+
     if (NULL == chooser->priv->monitors)
     {
         chooser->priv->selected = 0;
     }
+    else
+    {
+        chooser->priv->selected = 0;
+
+        for (id = 0; chooser->priv->monitors[id]; ++id)
+        {
+            monitors[id] = chooser->priv->monitors[id];
+        }
+        g_free (chooser->priv->monitors);
+    }
     
+    monitors[id] = monitor;
 
-    chooser->priv->monitors = g_slist_append (chooser->priv->monitors, monitor);
+    chooser->priv->monitors = monitors;
+    chooser->priv->n_monitors++;
 
-    return g_slist_index (chooser->priv->monitors, monitor);
+    return id;
 }
 
 gint
-rstto_monitor_chooser_set_pixbuf (
+rstto_monitor_chooser_set_image_surface (
         RsttoMonitorChooser *chooser,
         gint monitor_id,
-        GdkPixbuf *pixbuf,
+        cairo_surface_t *surface,
         GError **error)
 {
-    Monitor *monitor = g_slist_nth_data (chooser->priv->monitors, monitor_id);
+    Monitor *monitor;
     gint retval = -1;
 
+    g_return_val_if_fail (monitor_id < chooser->priv->n_monitors, retval);
+
+    monitor = chooser->priv->monitors[monitor_id];
+
     if (monitor)
     {
-        if (monitor->pixbuf)
+        if (monitor->image_surface)
         {
-            g_object_unref (monitor->pixbuf);
+            cairo_surface_destroy(monitor->image_surface);
         }
-        monitor->pixbuf = pixbuf;
+
+        monitor->image_surface = surface;
+
         retval = monitor_id;
     }
     if (GTK_WIDGET_REALIZED (GTK_WIDGET(chooser)))
@@ -580,14 +699,14 @@ cb_rstto_button_press_event (
     gint id = 0;
     gint width, height;
     
-    if (g_slist_length (chooser->priv->monitors) > 1)
+    if (chooser->priv->n_monitors > 1)
     {
-        row_width = sqrt (g_slist_length(chooser->priv->monitors));
+        row_width = sqrt (chooser->priv->n_monitors);
 
         width = widget->allocation.width*(0.4/((gdouble)row_width+1));
         height = width;
 
-        for (id = 0; id < (gint)g_slist_length(chooser->priv->monitors); ++id)
+        for (id = 0; id < chooser->priv->n_monitors; ++id)
         {
             if ( (event->x > ((gdouble)widget->allocation.width/2)+
                                 (((gdouble)widget->allocation.width/2)/
@@ -610,7 +729,10 @@ cb_rstto_button_press_event (
                 }
                 else
                 {
-                    chooser->priv->selected = id+1;
+                    if (id+1 != chooser->priv->n_monitors)
+                    {
+                        chooser->priv->selected = id+1;
+                    }
                 }
 
                 g_signal_emit (G_OBJECT (chooser), rstto_monitor_chooser_signals[RSTTO_MONITOR_CHOOSER_SIGNAL_CHANGED], 0, NULL);
@@ -629,21 +751,15 @@ rstto_monitor_chooser_get_selected (
     return chooser->priv->selected;
 }
 
-gboolean
-rstto_monitor_chooser_set_style (
+void
+rstto_monitor_chooser_get_dimensions (
         RsttoMonitorChooser *chooser,
-        gint monitor_id,
-        RsttoMonitorStyle style )
+        gint nr,
+        gint *width,
+        gint *height)
 {
-    GtkWidget *widget = GTK_WIDGET(chooser);
-    Monitor *monitor = g_slist_nth_data (chooser->priv->monitors, monitor_id);
-    g_return_val_if_fail (monitor != NULL, FALSE);
-    if ( NULL != monitor )
-    {
-        monitor->style = style;
-        if (GTK_WIDGET_REALIZED (widget))
-            rstto_monitor_chooser_paint (GTK_WIDGET(chooser));
-        return TRUE;
-    }
-    return FALSE;
+    g_return_if_fail (nr < chooser->priv->n_monitors);
+
+    *width = chooser->priv->monitors[nr]->width;
+    *height = chooser->priv->monitors[nr]->height;
 }
